@@ -35,14 +35,106 @@ TeleWindow::TeleWindow(Display *dpy)
 
     _bgPixmap = XCreatePixmap(_dpy, _rootWindow, 800, 480, depth);
 
+    printf("Loading background from '%s'\n", Settings::instance()->backgroundFilename());
+    printf("Background mode: %d\n", Settings::instance()->backgroundMode());
     Imlib_Image background = imlib_load_image(Settings::instance()->backgroundFilename());
     if (background == 0)
         printf("Cannot load background\n");
     else
     {
+        Imlib_Image bgToDraw = 0;
+
+        int bgXpos = 0;
+        int bgYpos = 0;
+
         imlib_context_set_image(background);
+        switch (Settings::instance()->backgroundMode())
+        {
+            case Settings::Stretched:
+            {
+                bgToDraw = imlib_create_cropped_scaled_image(
+                    0, 0,
+                    imlib_image_get_width(), imlib_image_get_height(),
+                    800, 480
+                );
+                break;
+            }
+
+            case Settings::Centered:
+            {
+                // Hildon's logic is not so easy in this case
+
+//                bgToDraw = imlib_create_cropped_image(
+//                    (imlib_image_get_width() - 800) / 2,
+//                    (imlib_image_get_height()- 480) / 2,
+//                    800, 480
+//                );
+
+                double scalex = 1.0;
+                double scaley = 1.0;
+
+                if (imlib_image_get_width() > 800 * 2)
+                    scalex = (double)imlib_image_get_width() / 2.0 / 800;
+                if (imlib_image_get_height() > 480 * 2)
+                    scaley = (double)imlib_image_get_height() / 2.0 / 480;
+
+                double scale = scalex;
+                if (scaley > scale)
+                    scale = scaley;
+
+                int newwidth = (int)(scale * 800);
+                int newheight= (int)(scale * 480);
+
+                bgToDraw = imlib_create_cropped_scaled_image(
+                    (imlib_image_get_width() - newwidth) / 2,
+                    (imlib_image_get_height()- newheight)/ 2,
+                    newwidth, newheight,
+                    800, 480
+                );
+
+                break;
+            }
+
+            case Settings::Scaled: case Settings::Cropped:
+            {
+                double scalex = 800.0 / imlib_image_get_width();
+                double scaley = 480.0 / imlib_image_get_height();
+                double scale = scalex;
+
+                if (Settings::instance()->backgroundMode() == Settings::Scaled)
+                {
+                    if (scaley < scale) scale = scaley;
+                }
+                else
+                {
+                    if (scaley > scale) scale = scaley;
+                }
+
+                int newwidth = (int)(imlib_image_get_width() * scale);
+                int newheight= (int)(imlib_image_get_height() * scale);
+
+                bgToDraw = imlib_create_cropped_scaled_image(
+                    0, 0,
+                    imlib_image_get_width(), imlib_image_get_height(),
+                    newwidth, newheight
+                );
+
+                bgXpos = (800 - newwidth) / 2;
+                bgYpos = (480 - newheight) / 2;
+                break;
+            }
+
+            default:
+                printf("Unknown background mode: %d\n", Settings::instance()->backgroundMode());
+        }
+
+        imlib_free_image();
+
+        imlib_context_set_image(bgToDraw);
         imlib_context_set_drawable(_bgPixmap);
-        imlib_render_image_on_drawable(0, 0);
+
+        imlib_render_image_on_drawable(bgXpos, bgYpos);
+
         imlib_free_image();
     }
 
